@@ -46,8 +46,9 @@ class TargetKind(Enum):
 BUILD_TYPE_DEBUG = "debug"
 BUILD_TYPE_RELEASE = "release"
 
-TOOLCHAIN_MSVC16 = "msvc16"
-TOOLCHAIN_LLVM13 = "llvm13"
+TOOLCHAIN_DEFAULT = "default"
+TOOLCHAIN_VS16_MSVC = "vs16_msvc"
+TOOLCHAIN_VS16_LLVM13 = "vs16_llvm13"
 
 OS_WINDOWS = "windows"
 
@@ -83,7 +84,7 @@ class ProcessRunnerDefault(ProcessRunner):
 
 class SystemConfigManifestProvider(ABC):
     @abstractmethod
-    def run(self, working_dir_abs_path):
+    def run(self, working_dir_abs_path, build_types):
         pass
 
     @abstractmethod
@@ -107,6 +108,6 @@ class BuildScriptEmitter(ABC):
         pass
 
     @abstractmethod
-    def contents(self, source_dir_abs_path, configure_dir_abs_path, build_dir_abs_path,
+    def contents(self, wks_name, source_dir_abs_path, configure_dir_abs_path, build_dir_abs_path,
                  toolchains_manifest, toolchains_settings, build_types, targets_names, targets_impls) -> str:
         pass
