@@ -117,6 +117,12 @@ class BFFEmitter:
         yield
         self._end_function()
 
+    @contextmanager
+    def remove_dir(self, name: str):
+        self._begin_function("RemoveDir", name)
+        yield
+        self._end_function()
+
     @contextmanager
     def compiler(self, name: str):
         self._begin_function("Compiler", name)
@@ -202,6 +208,29 @@ class BFFEmitter:
         return f(value)
 
 
+def get_target_kind_extension(target_kind):
+    if target_kind == base.TargetKind.STATIC_LIB:
+        return "lib"
+    elif target_kind == base.TargetKind.DYNAMIC_LIB:
+        return "dll"
+    elif target_kind == base.TargetKind.EXECUTABLE:
+        return "exe"
+    else:
+        raise NotImplementedError("Target kind of this type is not implemented.")
+
+
+def format_target_filename(target_name, target_kind):
+    ext = get_target_kind_extension(target_kind)
+    if len(ext) > 0:
+        return f"{target_name}.{ext}"
+    else:
+        return target_name
+
+
+def format_target_glob_pattern(target_kind):
+    return format_target_filename("*", target_kind)
+
+
 def format_compiler_node_name(toolchain_name: str) -> str:
     return f"compiler-{toolchain_name}"
 
@@ -214,7 +243,7 @@ def format_obj_node_name(target_node_name: str):
     return f"obj-{target_node_name}"
 
 
-def format_build_node_name(target_node_name: str):
+def format_make_node_name(target_node_name: str):
     return f"make-{target_node_name}"
 
 
@@ -268,6 +297,10 @@ class CompilerArgFormatter(abc.ABC):
     def format_include_dir_arg(self, include_dir: str) -> str:
         pass
 
+    @abc.abstractmethod
+    def format_system_include_dir_arg(self, include_dir: str) -> str:
+        pass
+
 
 class CompilerArgFormatterMSVC(CompilerArgFormatter):
     def format_input_output_args(self, output_dir_path: str, output_base_name) -> list[str]:
@@ -283,6 +316,9 @@ class CompilerArgFormatterMSVC(CompilerArgFormatter):
     def format_include_dir_arg(self, include_dir: str) -> str:
         return f"/I\"{include_dir}\""
 
+    def format_system_include_dir_arg(self, include_dir: str) -> str:
+        return f"/I\"{include_dir}\""
+
 
 class CompilerArgFormatterClang(CompilerArgFormatter):
     def format_input_output_args(self, output_dir_path: str, output_base_name) -> list[str]:
@@ -297,6 +333,9 @@ class CompilerArgFormatterClang(CompilerArgFormatter):
     def format_include_dir_arg(self, include_dir: str) -> str:
         return f"-I\"{include_dir}\""
 
+    def format_system_include_dir_arg(self, include_dir: str) -> str:
+        return f"-isystem\"{include_dir}\""
+
 
 class CompilerArgFormatterGCC(CompilerArgFormatterClang):
     pass
@@ -324,7 +363,7 @@ def build_compiler_options(
 ):
     args = arg_formatter.format_input_output_args(output_dir_path, output_base_name)
     args += base_args
-    args += [arg_formatter.format_include_dir_arg(include_dir) for include_dir in base_include_dirs]
+    args += [arg_formatter.format_system_include_dir_arg(include_dir) for include_dir in base_include_dirs]
     args += [arg_formatter.format_definition_compiler_arg(name, value) for name, value in base_definitions.items()]
     args += [arg_formatter.format_include_dir_arg(include_dir) for include_dir in target_include_dirs]
     args += [arg_formatter.format_definition_compiler_arg(name, value) for name, value in target_definitions.items()]
@@ -483,14 +522,17 @@ def build_linker_options(
 ):
     if output_target_kind != base.TargetKind.DYNAMIC_LIB and output_target_kind != base.TargetKind.EXECUTABLE:
         raise RuntimeError()
-    
+
     args = arg_formatter.format_input_output_declaration(output_dir_path, output_base_name, output_target_kind)
     args += base_args
+
     args += [arg_formatter.format_lib_arg(lib) for lib in base_link_libs]
     args += [arg_formatter.format_lib_dir_arg(lib_dir) for lib_dir in base_lib_dirs]
-    # External libs are added manully, not through "Libraries" variable.
+
+    # External libs are added to commandl ine manully, not through "Libraries" variable.
     # If the extension is skipped, fastbuild tries to use "obj"
-    args += [arg_formatter.format_lib_arg(f"{lib}.lib") for lib in external_link_libs]
+    args += [arg_formatter.format_lib_arg(f"{lib}.{get_target_kind_extension(base.TargetKind.STATIC_LIB)}")
+             for lib in external_link_libs]
     args += [arg_formatter.format_lib_dir_arg(lib_dir) for lib_dir in external_lib_dirs]
 
     return args
@@ -498,20 +540,38 @@ def build_linker_options(
 
 class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
     ALL_TARGETS_TARGET_NAME = "rapid_all"
+    SLN_TARGET_NAME = "rapid_sln"
+    CLEAN_TARGET_NAME = "rapid_clean"
 
     def filename(self):
         return "fbuild.bff"
 
-    # noinspection PyUnreachableCode
-    def contents(self, source_dir_abs_path, configure_dir_abs_path, build_dir_abs_path,
+    def contents(self, wks_name, source_dir_abs_path, configure_dir_abs_path, build_dir_abs_path,
                  toolchains_manifest, toolchains_settings, build_types, targets_names, targets_impls) -> str:
 
+        assert (set(toolchains_manifest.keys()) == set(toolchains_settings.keys()))
+
         intermediate_dir_abs_path = fs.path.join(build_dir_abs_path, "intermediate")
         deploy_dir_abs_path = fs.path.join(build_dir_abs_path, "bin")
 
         out_bff = BFFEmitter()
         out_bff.once()
 
+        self._emit_compilers(out_bff, toolchains_manifest)
+        self._emit_compilation_targets(out_bff, intermediate_dir_abs_path, deploy_dir_abs_path, toolchains_manifest,
+                                       toolchains_settings, build_types, targets_names, targets_impls)
+        self._emit_vs_solution_targets(out_bff, wks_name, source_dir_abs_path, configure_dir_abs_path, deploy_dir_abs_path,
+                                       toolchains_manifest, build_types, targets_names, targets_impls)
+
+        with out_bff.remove_dir(self.CLEAN_TARGET_NAME):
+            out_bff.emit_var_assignment("RemovePaths", build_dir_abs_path)
+            out_bff.emit_var_assignment("RemovePathsRecurse", True)
+            
+        return out_bff.build()
+
+    @staticmethod
+    def _emit_compilers(out_bff, toolchains_manifest):
+        
         for toolchain_name, toolchain_def in toolchains_manifest.items():
             with out_bff.compiler(format_compiler_node_name(toolchain_name)):
                 compiler_path = toolchain_def["compiler_path"]
@@ -521,6 +581,9 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                 out_bff.emit_var_assignment("CompilerFamily", compiler_family_to_fastbuild_value(compiler_family))
                 out_bff.emit_var_assignment("ExtraFiles", toolchain_def["compiler_extra_files"])
 
+    def _emit_compilation_targets(self, out_bff, intermediate_dir_abs_path, deploy_dir_abs_path, toolchains_manifest,
+                                  toolchains_settings, build_types, targets_names, targets_impls):
+
         all_nodes: list[str] = []
         per_toolchain_nodes: dict[str, list[str]] = {
             toolchain_name: [] for toolchain_name in toolchains_manifest.keys()
@@ -533,6 +596,7 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                 build_type: [] for build_type in build_types
             } for toolchain_name in toolchains_manifest.keys()
         }
+        external_deploy_nodes = set()
 
         for toolchain_name, toolchain_def in toolchains_manifest.items():
 
@@ -543,7 +607,7 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
             compiler_arg_formatter = create_compiler_arg_formatter(toolchain_compiler_family)
             librarian_arg_formatter = create_librarian_arg_formatter(toolchain_librarian_family)
             linker_arg_formatter = create_linker_arg_formatter(toolchain_linker_family)
-            
+
             toolchain_include_dirs = toolchain_def["toolchain_include_dirs"]
             toolchain_lib_dirs = toolchain_def["toolchain_lib_dirs"]
 
@@ -578,13 +642,15 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                     )
                     compiler_input_path = target_impl["source_dir"]
                     compiler_input_pattern = ["*.c", "*.cpp", "*.cxx"]
-                    
+
+                    make_node_name = format_make_node_name(target_node_name)
+                    deploy_node_name = format_deploy_node_name(target_node_name)
+
                     if target_kind == base.TargetKind.STATIC_LIB:
 
-                        librarian_options = build_librarian_options(librarian_arg_formatter)
-                        librarian_output_path = fs.path.join(target_output_dir_path, f"{target_node_name}.lib")
+                        librarian_output_path = fs.path.join(target_output_dir_path,
+                                                             format_target_filename(target_node_name, target_kind))
 
-                        make_node_name = format_build_node_name(target_node_name)
                         with out_bff.library(make_node_name):
                             out_bff.emit_var_assignment("Hidden", True)
                             out_bff.emit_var_assignment("Compiler", format_compiler_node_name(toolchain_name))
@@ -597,11 +663,11 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                             out_bff.line_break()
                             out_bff.emit_var_assignment("Librarian", toolchain_def["librarian_path"])
                             out_bff.emit_var_assignment("LibrarianOutput", librarian_output_path)
-                            out_bff.emit_var_assignment("LibrarianOptions", " ".join(librarian_options),
+                            out_bff.emit_var_assignment("LibrarianOptions",
+                                                        " ".join(build_librarian_options(librarian_arg_formatter)),
                                                         should_split_str=True)
-
+                            
                         # Deploy step does nothing for static libs.
-                        deploy_node_name = format_deploy_node_name(target_node_name)
                         with out_bff.alias(deploy_node_name):
                             out_bff.emit_var_assignment("Hidden", True)
                             out_bff.emit_var_assignment("Targets", make_node_name)
@@ -609,7 +675,7 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                         with out_bff.alias(target_node_name):
                             out_bff.emit_var_assignment("Targets", deploy_node_name)
 
-                    elif target_kind == base.TargetKind.DYNAMIC_LIB:
+                    elif target_kind == base.TargetKind.DYNAMIC_LIB or target_kind == base.TargetKind.EXECUTABLE:
 
                         object_list_node_name = format_obj_node_name(target_node_name)
                         with out_bff.object_list(object_list_node_name):
@@ -623,10 +689,11 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                                                         should_split_str=True)
 
                         #
-                        linker_output_path = fs.path.join(target_output_dir_path, f"{target_node_name}.dll")
+                        linker_output_path = fs.path.join(target_output_dir_path,
+                                                          format_target_filename(target_node_name, target_kind))
 
                         libraries = [object_list_node_name]
-                        libraries += [format_build_node_name(format_target_node_name(lib, toolchain_name, build_type))
+                        libraries += [format_make_node_name(format_target_node_name(lib, toolchain_name, build_type))
                                       for lib in target_impl["link_libs"]]
 
                         linker_options = build_linker_options(
@@ -636,92 +703,61 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                             target_impl["link_libs_external"], target_impl["link_libs_external_dirs"]
                         )
 
-                        make_node_name = format_build_node_name(target_node_name)
-                        with out_bff.dll(make_node_name):
-                            # out_bff.emit_var_assignment("Hidden", True)  # DLL does not support .Hidden
-                            out_bff.emit_var_assignment("Libraries", libraries)
-                            out_bff.emit_var_assignment("Linker", toolchain_def["linker_path"])
-                            out_bff.emit_var_assignment("LinkerOutput", linker_output_path)
-                            out_bff.emit_var_assignment("LinkerOptions", " ".join(linker_options),
-                                                        should_split_str=True)
+                        if target_kind == base.TargetKind.DYNAMIC_LIB:
+                            with out_bff.dll(make_node_name):
+                                # out_bff.emit_var_assignment("Hidden", True)  # DLL does not support .Hidden
+                                out_bff.emit_var_assignment("Libraries", libraries)
+                                out_bff.emit_var_assignment("Linker", toolchain_def["linker_path"])
+                                out_bff.emit_var_assignment("LinkerOutput", linker_output_path)
+                                out_bff.emit_var_assignment("LinkerOptions", " ".join(linker_options),
+                                                            should_split_str=True)
+                                
+                        elif target_kind == base.TargetKind.EXECUTABLE:
+                            with out_bff.exe(make_node_name):
+                                # out_bff.emit_var_assignment("Hidden", True)  # Executable does not support .Hidden
+                                out_bff.emit_var_assignment("Libraries", libraries)
+                                out_bff.emit_var_assignment("Linker", toolchain_def["linker_path"])
+                                out_bff.emit_var_assignment("LinkerOutput", linker_output_path)
+                                out_bff.emit_var_assignment("LinkerOptions", " ".join(linker_options),
+                                                            should_split_str=True)
+                                
+                        deploy_deps = [make_node_name]
+                        deploy_deps += [
+                            format_deploy_node_name(format_target_node_name(lib, toolchain_name, build_type))
+                            for lib in target_impl["load_time_libs"]]
 
-                        #
-                        prebuild_deps = [make_node_name]
-                        prebuild_deps += [format_deploy_node_name(format_target_node_name(lib, toolchain_name, build_type))
-                                          for lib in target_impl["runtime_libs"]]
+                        for path in target_impl["load_time_libs_external"]:
+                            deploy_external = format_deploy_node_name(path)
+                            if path not in external_deploy_nodes:
+                                external_deploy_nodes.add(path)
+
+                                with out_bff.copy_dir(deploy_external):
+                                    out_bff.emit_var_assignment("Hidden", True)
+                                    out_bff.emit_var_assignment("PreBuildDependencies", [])
+                                    out_bff.emit_var_assignment("SourcePaths", fs.path.dirname(path))
+                                    out_bff.emit_var_assignment("SourcePathsPattern", [(fs.path.basename(path))])
+                                    out_bff.emit_var_assignment("Dest", deploy_dir_abs_path)
+                            deploy_deps.append(deploy_external)
 
-                        deploy_node_name = format_deploy_node_name(target_node_name)
                         with out_bff.copy_dir(deploy_node_name):
                             out_bff.emit_var_assignment("Hidden", True)
-                            out_bff.emit_var_assignment("PreBuildDependencies", prebuild_deps)
+                            out_bff.emit_var_assignment("PreBuildDependencies", deploy_deps)
                             out_bff.emit_var_assignment("SourcePaths", target_output_dir_path)
-                            out_bff.emit_var_assignment("SourcePathsPattern", ["*.dll", "*.pdb"])
+                            out_bff.emit_var_assignment("SourcePathsPattern",
+                                                        [format_target_glob_pattern(target_kind), "*.pdb"])
                             out_bff.emit_var_assignment("Dest", deploy_dir_abs_path)
 
                         with out_bff.alias(target_node_name):
                             out_bff.emit_var_assignment("Targets", deploy_node_name)
-
-                    elif target_kind == base.TargetKind.EXECUTABLE:
-
-                        object_list_node_name = format_obj_node_name(target_node_name)
-
-                        with out_bff.object_list(object_list_node_name):
-                            out_bff.emit_var_assignment("Hidden", True)
-                            out_bff.emit_var_assignment("Compiler", format_compiler_node_name(toolchain_name))
-                            out_bff.emit_var_assignment("CompilerInputPath", compiler_input_path)
-                            out_bff.emit_var_assignment("CompilerInputPathRecurse", True)
-                            out_bff.emit_var_assignment("CompilerInputPattern", compiler_input_pattern)
-                            out_bff.emit_var_assignment("CompilerOutputPath", target_output_dir_path)
-                            out_bff.emit_var_assignment("CompilerOptions", " ".join(compiler_options),
-                                                        should_split_str=True)
-
-                        #
-                        linker_output_path = fs.path.join(target_output_dir_path, f"{target_node_name}.exe")
-
-                        libraries = [object_list_node_name]
-                        libraries += [format_build_node_name(format_target_node_name(lib, toolchain_name, build_type))
-                                      for lib in target_impl["link_libs"]]
-
-                        linker_options = build_linker_options(
-                            linker_arg_formatter, target_output_dir_path, target_node_name, target_kind,
-                            toolchain_build_type_linker_args, toolchain_build_type_link_libs, toolchain_lib_dirs,
-                            target_impl["link_libs_external"], target_impl["link_libs_external_dirs"]
-                        )
-
-                        make_node_name = format_build_node_name(target_node_name)
-                        with out_bff.exe(make_node_name):
-                            # out_bff.emit_var_assignment("Hidden", True)  # Executable does not support .Hidden
-                            out_bff.emit_var_assignment("Libraries", libraries)
-                            out_bff.emit_var_assignment("Linker", toolchain_def["linker_path"])
-                            out_bff.emit_var_assignment("LinkerOutput", linker_output_path)
-                            out_bff.emit_var_assignment("LinkerOptions", " ".join(linker_options),
-                                                        should_split_str=True)
-
-                        #
-                        prebuild_deps = [make_node_name]
-                        prebuild_deps += [format_deploy_node_name(format_target_node_name(lib, toolchain_name, build_type))
-                                          for lib in target_impl["runtime_libs"]]
-
-                        deploy_node_name = format_deploy_node_name(target_node_name)
-                        with out_bff.copy_dir(deploy_node_name):
-                            # CopyDir does not support .Hidden, however it is not listed with -showtargets.
-                            # out_bff.emit_var_assignment("Hidden", True)
-                            out_bff.emit_var_assignment("PreBuildDependencies", prebuild_deps)
-                            out_bff.emit_var_assignment("SourcePaths", target_output_dir_path)
-                            out_bff.emit_var_assignment("SourcePathsPattern", ["*.exe", "*.pdb"])
-                            out_bff.emit_var_assignment("Dest", deploy_dir_abs_path)
-
-                        with out_bff.alias(target_node_name):
-                            out_bff.emit_var_assignment("Targets", deploy_node_name)
-
+                            
                     else:
-                        raise RuntimeError("unsupported target kind")
+                        raise NotImplementedError("unsupported target kind")
 
                     all_nodes.append(target_node_name)
                     per_build_type_nodes[build_type].append(target_node_name)
                     per_toolchain_nodes[toolchain_name].append(target_node_name)
                     config_vector_nodes[toolchain_name][build_type].append(target_node_name)
-
+                    
         for build_type, nodes in per_build_type_nodes.items():
             with out_bff.alias(f"{self.ALL_TARGETS_TARGET_NAME}-{build_type}"):
                 out_bff.emit_var_assignment("Targets", nodes)
@@ -738,8 +774,11 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
         with out_bff.alias(f"{self.ALL_TARGETS_TARGET_NAME}"):
             out_bff.emit_var_assignment("Targets", all_nodes)
 
-        # vc projects and sln
-        bff_path = fs.path.join(f"{configure_dir_abs_path}", "fbuild.bff")
+    # noinspection PyUnreachableCode
+    def _emit_vs_solution_targets(self, out_bff, wks_name, source_dir_abs_path, configure_dir_abs_path, deploy_dir_abs_path,
+                                  toolchains_manifest, build_types, targets_names, targets_impls):
+
+        bff_path = fs.path.join(f"{configure_dir_abs_path}", self.filename())
         vs_dir_abs_path = fs.path.join(configure_dir_abs_path, "vs")
 
         project_nodes = []
@@ -768,10 +807,11 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
                             cfg["Target"] = target_node_name
                             cfg["ProjectBuildCommand"] = f"fbuild -ide -config \"{bff_path}\" {target_node_name}"
-                            cfg["ProjectRebuildCommand"] = f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}"
+                            cfg[
+                                "ProjectRebuildCommand"] = f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}"
                             # ProjectCleanCommand
 
-                            cfg["LocalDebuggerCommand"] = fs.path.join(build_dir_abs_path, f"{target_node_name}.exe")
+                            cfg["LocalDebuggerCommand"] = fs.path.join(deploy_dir_abs_path, f"{target_node_name}.exe")
 
                             include_dirs_intellisense = ""
                             for include_dir in target_impl["include_dirs"]:
@@ -802,13 +842,16 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                             out_bff.emit_var_assignment("Platform", "x64")
                             out_bff.emit_var_assignment("PlatformToolset", "v142")
                             out_bff.emit_var_assignment("Config", f"{toolchain_name}-{build_type}")
-                            #out_bff.emit_var_assignment("Config", "Debug")
+                            # out_bff.emit_var_assignment("Config", "Debug")
                             out_bff.line_break()
                             out_bff.emit_var_assignment("Target", target_node_name)
-                            out_bff.emit_var_assignment("ProjectBuildCommand", f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
-                            out_bff.emit_var_assignment("ProjectRebuildCommand", f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
+                            out_bff.emit_var_assignment("ProjectBuildCommand",
+                                                        f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
+                            out_bff.emit_var_assignment("ProjectRebuildCommand",
+                                                        f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
                             # ProjectCleanCommand
-                            out_bff.emit_var_assignment("LocalDebuggerCommand", fs.path.join(build_dir_abs_path, f"{target_node_name}.exe"))
+                            out_bff.emit_var_assignment("LocalDebuggerCommand",
+                                                        fs.path.join(deploy_dir_abs_path, f"{target_node_name}.exe"))
                             out_bff.line_break()
 
                             include_dirs_intellisense = ""
@@ -828,7 +871,6 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
                             out_bff._emit_line(".ProjectConfigs + .cfg")
 
-
             project_nodes.append(project_node_name)
 
         # special 'all' project
@@ -842,7 +884,8 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                 project_configs = []
                 for toolchain_name in toolchains_manifest.keys():
                     for build_type in build_types:
-                        target_node_name = format_target_node_name(self.ALL_TARGETS_TARGET_NAME, toolchain_name, build_type)
+                        target_node_name = format_target_node_name(self.ALL_TARGETS_TARGET_NAME, toolchain_name,
+                                                                   build_type)
 
                         cfg["Platform"] = "x64"
                         cfg["PlatformToolset"] = "v142"
@@ -861,7 +904,8 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
                 for toolchain_name in toolchains_manifest.keys():
                     for build_type in build_types:
-                        target_node_name = format_target_node_name(self.ALL_TARGETS_TARGET_NAME, toolchain_name, build_type)
+                        target_node_name = format_target_node_name(self.ALL_TARGETS_TARGET_NAME, toolchain_name,
+                                                                   build_type)
 
                         out_bff._emit_line(".cfg = [")
                         out_bff._indent()
@@ -869,11 +913,13 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
                         out_bff.emit_var_assignment("Platform", "x64")
                         out_bff.emit_var_assignment("PlatformToolset", "v142")
                         out_bff.emit_var_assignment("Config", f"{toolchain_name}-{build_type}")
-                        #out_bff.emit_var_assignment("Config", "Debug")
+                        # out_bff.emit_var_assignment("Config", "Debug")
                         out_bff.line_break()
                         out_bff.emit_var_assignment("Target", target_node_name)
-                        out_bff.emit_var_assignment("ProjectBuildCommand", f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
-                        out_bff.emit_var_assignment("ProjectRebuildCommand", f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
+                        out_bff.emit_var_assignment("ProjectBuildCommand",
+                                                    f"fbuild -ide -config \"{bff_path}\" {target_node_name}")
+                        out_bff.emit_var_assignment("ProjectRebuildCommand",
+                                                    f"fbuild -ide -config \"{bff_path}\" -clean {target_node_name}")
                         # ProjectCleanCommand
                         out_bff.emit_var_assignment("LocalDebuggerCommand", "")
 
@@ -884,9 +930,9 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
         project_nodes.append(project_node_name)
 
         # solution
-        sln_path = fs.path.join(vs_dir_abs_path, "generated.sln")
+        sln_path = fs.path.join(vs_dir_abs_path, f"{wks_name}.sln")
 
-        with out_bff.vssolution("rapid_sln"):
+        with out_bff.vssolution(self.SLN_TARGET_NAME):
             out_bff.emit_var_assignment("SolutionOutput", sln_path)
             out_bff.emit_var_assignment("SolutionProjects", project_nodes)
             out_bff.line_break()
@@ -921,7 +967,6 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
                 for toolchain_name in toolchains_manifest.keys():
                     for build_type in build_types:
-
                         out_bff._emit_line(".cfg = [")
                         out_bff._indent()
 
@@ -951,5 +996,3 @@ class BuildScriptEmitterFastbuild(base.BuildScriptEmitter):
 
             out_bff.emit_var_assignment("SolutionVisualStudioVersion", "15.0")
             out_bff.emit_var_assignment("SolutionMinimumVisualStudioVersion", "15.0")
-
-        return out_bff.build()
