@@ -85,6 +85,12 @@ def append_unique(list_: list[Any], item: any):
         list_.append(item)
 
 
+def extend_unique(list_a: list[Any], list_b: list[Any]):
+    for item in list_b:
+        if item not in list_a:
+            list_a.append(item)
+
+
 def _validate_third_party_manifest(manifest):
     if not isinstance(manifest, dict):
         return False
@@ -99,17 +105,25 @@ def _validate_toolchains_manifest(manifest):
     return True
 
 
-def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchains_manifest, build_types,
+def _validate_toolchains_settings(settings):
+    if not isinstance(settings, dict):
+        return False
+    if len(settings.keys()) == 0:
+        return False
+    return True
+
+
+def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchains, build_types,
                                                    third_party_manifest, workspace_def: WorkspaceDefinition):
     assert(len(source_dir_abs_path) != 0)
     assert(len(build_types) != 0)
-    assert(_validate_toolchains_manifest(toolchains_manifest))
+    assert(len(toolchains) != 0)
     assert(_validate_third_party_manifest(third_party_manifest))
 
-    targets_interfaces = dict(third_party_manifest)
+    targets_interfaces = {}
     targets_impls = {}
 
-    for toolchain_name in toolchains_manifest.keys():
+    for toolchain_name in toolchains:
         for build_type in build_types:
             for target_def in workspace_def.targets:
                 iface = {
@@ -119,7 +133,8 @@ def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchai
                     "link_libs": [],
                     "link_libs_external": [],
                     "link_libs_external_dirs": [],
-                    "runtime_libs": []
+                    "load_time_libs": [],
+                    "load_time_libs_external": []
                 }
 
                 impl = {
@@ -130,7 +145,8 @@ def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchai
                     "link_libs": [],
                     "link_libs_external": [],
                     "link_libs_external_dirs": [],
-                    "runtime_libs": []
+                    "load_time_libs": [],
+                    "load_time_libs_external": []
                 }
 
                 target_public_include_dir_name = "include"
@@ -164,41 +180,12 @@ def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchai
                     if name not in impl["definitions"]:
                         impl["definitions"][name] = value
 
-                for dependency in target_def.public_props.dependencies:
-                    dep_iface = targets_interfaces[base.build_target_key(dependency, toolchain_name, build_type)]
+                for dep_name in target_def.public_props.dependencies:
+                    dep_iface = targets_interfaces[base.build_target_key(dep_name, toolchain_name, build_type)]
 
                     if dep_iface["kind"] == base.TargetKind.EXECUTABLE:
                         raise RuntimeError("Cannot declare dep on an exe.")
 
-                    for link_lib in dep_iface["link_libs"]:
-                        append_unique(iface["link_libs"], link_lib)
-                        append_unique(impl["link_libs"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external"]:
-                        append_unique(iface["link_libs_external"], link_lib)
-                        append_unique(impl["link_libs_external"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external_dirs"]:
-                        append_unique(iface["link_libs_external_dirs"], link_lib)
-                        append_unique(impl["link_libs_external_dirs"], link_lib)
-
-                for dependency in target_def.public_props.externals:
-                    dep_iface = targets_interfaces[
-                        base.build_target_key(dependency, toolchain_name, build_type)]
-
-                    for link_lib in dep_iface["link_libs_external"]:
-                        append_unique(iface["link_libs_external"], link_lib)
-                        append_unique(impl["link_libs_external"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external_dirs"]:
-                        append_unique(iface["link_libs_external_dirs"], link_lib)
-                        append_unique(impl["link_libs_external_dirs"], link_lib)
-
-                deps = target_def.public_props.dependencies + target_def.public_props.externals
-
-                for dependency in deps:
-                    dep_iface = targets_interfaces[base.build_target_key(dependency, toolchain_name, build_type)]
-
                     for name, value in dep_iface["definitions"].items():
                         # TODO: resolve conflicts (A=3, A=4)
                         if name not in iface["definitions"]:
@@ -206,78 +193,115 @@ def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchai
                         if name not in impl["definitions"]:
                             impl["definitions"][name] = value
 
-                    for d in dep_iface["include_dirs"]:
-                        append_unique(iface["include_dirs"], d)
-                        append_unique(impl["include_dirs"], d)
+                    extend_unique(iface["include_dirs"], dep_iface["include_dirs"])
+                    extend_unique(impl["include_dirs"], dep_iface["include_dirs"])
 
-                    for d in dep_iface["runtime_libs"]:
-                        append_unique(iface["runtime_libs"], d)
-                        # TODO: Is that a typo?
-                        append_unique(impl["runtime_libs"], d)
+                    extend_unique(iface["link_libs"], dep_iface["link_libs"])
+                    extend_unique(impl["link_libs"], dep_iface["link_libs"])
+
+                    extend_unique(iface["link_libs_external"], dep_iface["link_libs_external"])
+                    extend_unique(impl["link_libs_external"], dep_iface["link_libs_external"])
+
+                    extend_unique(iface["link_libs_external_dirs"], dep_iface["link_libs_external_dirs"])
+                    extend_unique(impl["link_libs_external_dirs"], dep_iface["link_libs_external_dirs"])
 
                     if dep_iface["kind"] == base.TargetKind.DYNAMIC_LIB:
-                        append_unique(iface["runtime_libs"], dependency)
-                        append_unique(impl["runtime_libs"], dependency)
+                        append_unique(iface["load_time_libs"], dep_name)
+                        append_unique(impl["load_time_libs"], dep_name)
+
+                    extend_unique(iface["load_time_libs"], dep_iface["load_time_libs"])
+                    extend_unique(impl["load_time_libs"], dep_iface["load_time_libs"])
+
+                    extend_unique(iface["load_time_libs_external"], dep_iface["load_time_libs_external"])
+                    extend_unique(impl["load_time_libs_external"], dep_iface["load_time_libs_external"])
+
+                for ext_name in target_def.public_props.externals:
+                    dep_iface = third_party_manifest[base.build_target_key(ext_name, base.TOOLCHAIN_DEFAULT, build_type)]
+
+                    for name, value in dep_iface["definitions"].items():
+                        # TODO: resolve conflicts (A=3, A=4)
+                        if name not in iface["definitions"]:
+                            iface["definitions"][name] = value
+                        if name not in impl["definitions"]:
+                            impl["definitions"][name] = value
+
+                    extend_unique(iface["include_dirs"], dep_iface["include_dirs"])
+                    extend_unique(impl["include_dirs"], dep_iface["include_dirs"])
+
+                    extend_unique(iface["link_libs_external"], dep_iface["link_libs"])
+                    extend_unique(impl["link_libs_external"], dep_iface["link_libs"])
+
+                    extend_unique(iface["link_libs_external_dirs"], dep_iface["link_libs_dirs"])
+                    extend_unique(impl["link_libs_external_dirs"], dep_iface["link_libs_dirs"])
+
+                    extend_unique(iface["load_time_libs_external"], dep_iface["load_time_libs"])
+                    extend_unique(impl["load_time_libs_external"], dep_iface["load_time_libs"])
 
                 # process private section
 
                 for name, value in target_def.private_props.definitions.items():
                     # TODO: resolve conflicts (A=3, A=4)
+                    # if name not in iface["definitions"]:
+                    #    iface["definitions"][name] = value
                     if name not in impl["definitions"]:
                         impl["definitions"][name] = value
 
-                for dependency in target_def.private_props.dependencies:
-                    dep_iface = targets_interfaces[base.build_target_key(dependency, toolchain_name, build_type)]
+                for dep_name in target_def.private_props.dependencies:
+                    dep_iface = targets_interfaces[base.build_target_key(dep_name, toolchain_name, build_type)]
 
                     if dep_iface["kind"] == base.TargetKind.EXECUTABLE:
                         raise RuntimeError("Cannot declare dep on an exe.")
 
-                    for link_lib in dep_iface["link_libs"]:
-                        append_unique(iface["link_libs"], link_lib)
-                        append_unique(impl["link_libs"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external"]:
-                        append_unique(iface["link_libs_external"], link_lib)
-                        append_unique(impl["link_libs_external"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external_dirs"]:
-                        append_unique(iface["link_libs_external_dirs"], link_lib)
-                        append_unique(impl["link_libs_external_dirs"], link_lib)
-
-                for dependency in target_def.private_props.externals:
-                    dep_iface = targets_interfaces[base.build_target_key(dependency, toolchain_name, build_type)]
-
-                    for link_lib in dep_iface["link_libs_external"]:
-                        append_unique(iface["link_libs_external"], link_lib)
-                        append_unique(impl["link_libs_external"], link_lib)
-
-                    for link_lib in dep_iface["link_libs_external_dirs"]:
-                        append_unique(iface["link_libs_external_dirs"], link_lib)
-                        append_unique(impl["link_libs_external_dirs"], link_lib)
-
-                deps = target_def.private_props.dependencies + target_def.private_props.externals
-
-                for dependency in deps:
-                    dep_iface = targets_interfaces[
-                        base.build_target_key(dependency, toolchain_name, build_type)]
-
-                    # print(json.dumps(dep_iface, indent=4))
-
                     for name, value in dep_iface["definitions"].items():
                         # TODO: resolve conflicts (A=3, A=4)
+                        # if name not in iface["definitions"]:
+                        #    iface["definitions"][name] = value
                         if name not in impl["definitions"]:
                             impl["definitions"][name] = value
 
-                    for d in dep_iface["include_dirs"]:
-                        append_unique(impl["include_dirs"], d)
+                    # extend_unique(iface["include_dirs"], dep_iface["include_dirs"])
+                    extend_unique(impl["include_dirs"], dep_iface["include_dirs"])
 
-                    for d in dep_iface["runtime_libs"]:
-                        append_unique(iface["runtime_libs"], d)
-                        append_unique(impl["runtime_libs"], d)
+                    extend_unique(iface["link_libs"], dep_iface["link_libs"])
+                    extend_unique(impl["link_libs"], dep_iface["link_libs"])
+
+                    extend_unique(iface["link_libs_external"], dep_iface["link_libs_external"])
+                    extend_unique(impl["link_libs_external"], dep_iface["link_libs_external"])
+
+                    extend_unique(iface["link_libs_external_dirs"], dep_iface["link_libs_external_dirs"])
+                    extend_unique(impl["link_libs_external_dirs"], dep_iface["link_libs_external_dirs"])
 
                     if dep_iface["kind"] == base.TargetKind.DYNAMIC_LIB:
-                        append_unique(iface["runtime_libs"], dependency)
-                        append_unique(impl["runtime_libs"], dependency)
+                        append_unique(iface["load_time_libs"], dep_name)
+                        append_unique(impl["load_time_libs"], dep_name)
+
+                    extend_unique(iface["load_time_libs"], dep_iface["load_time_libs"])
+                    extend_unique(impl["load_time_libs"], dep_iface["load_time_libs"])
+
+                    extend_unique(iface["load_time_libs_external"], dep_iface["load_time_libs_external"])
+                    extend_unique(impl["load_time_libs_external"], dep_iface["load_time_libs_external"])
+
+                for ext_name in target_def.private_props.externals:
+                    dep_iface = third_party_manifest[base.build_target_key(ext_name, base.TOOLCHAIN_DEFAULT, build_type)]
+                        
+                    for name, value in dep_iface["definitions"].items():
+                        # TODO: resolve conflicts (A=3, A=4)
+                        # if name not in iface["definitions"]:
+                        #     iface["definitions"][name] = value
+                        if name not in impl["definitions"]:
+                            impl["definitions"][name] = value
+
+                    # extend_unique(iface["include_dirs"], dep_iface["include_dirs"])
+                    extend_unique(impl["include_dirs"], dep_iface["include_dirs"])
+
+                    extend_unique(iface["link_libs_external"], dep_iface["link_libs"])
+                    extend_unique(impl["link_libs_external"], dep_iface["link_libs"])
+
+                    extend_unique(iface["link_libs_external_dirs"], dep_iface["link_libs_dirs"])
+                    extend_unique(impl["link_libs_external_dirs"], dep_iface["link_libs_dirs"])
+
+                    extend_unique(iface["load_time_libs_external"], dep_iface["load_time_libs"])
+                    extend_unique(impl["load_time_libs_external"], dep_iface["load_time_libs"])
 
                 # save result
                 target_key = base.build_target_key(target_def.name, toolchain_name, build_type)
@@ -292,6 +316,7 @@ def _create_targets_interfaces_and_implementations(source_dir_abs_path, toolchai
 
 class Workspace:
     def __init__(self, *,
+                 name: str,
                  wks_fs: FS,
                  logger: base.Logger,
                  system_config_provider: base.SystemConfigManifestProvider,
@@ -299,6 +324,8 @@ class Workspace:
                  build_script_emitter: base.BuildScriptEmitter
                  ):
 
+        # TODO: sanitize name string: only alphanum + _, no spaces
+        self.name = name
         self.fs = wks_fs
 
         self.configure_dir_name = "configure"
@@ -368,7 +395,7 @@ class Workspace:
         #
         self.fs.makedir(self.configure_dir_name, recreate=True)
 
-        self.system_config_provider.run(self._get_configure_dir_abs_path())
+        self.system_config_provider.run(self._get_configure_dir_abs_path(), self.build_types)
 
         self.third_party_manifest = self.system_config_provider.get_third_party_manifest()
 
@@ -381,13 +408,31 @@ class Workspace:
         if not _validate_toolchains_manifest(self.toolchains_manifest):
             self.logger.log_error("Provided toolchains manifest is invalid.")
             return False
+        
+        toolchains_settings = self.toolchain_settings_provider.get_toolchain_settings()
+
+        if not _validate_toolchains_settings(toolchains_settings):
+            self.logger.log_error("Provided toolchains settings format is invalid.")
+            return False
+
+        manifest_toolchains = set(self.toolchains_manifest.keys())
+        settings_toolchains = set(toolchains_settings.keys())
+        if not manifest_toolchains.issubset(settings_toolchains):
+            self.logger.log_error("Missing settings for toolchains.")
+            manifest_toolchains = list(manifest_toolchains)
+            manifest_toolchains.sort()
+            settings_toolchains = list(settings_toolchains)
+            settings_toolchains.sort()
+            self.logger.log_error(f"Toolchains in manifest: {manifest_toolchains}")
+            self.logger.log_error(f"Toolchains for which settings were provided : {settings_toolchains}")
+            return False
 
         #
         # Build targets implementations
         #
         targets_interfaces, targets_impls = _create_targets_interfaces_and_implementations(
             self._get_source_dir_abs_path(),
-            self.toolchains_manifest,
+            self.toolchains_manifest.keys(),
             self.build_types,
             self.third_party_manifest,
             self.workspace_def
@@ -402,11 +447,12 @@ class Workspace:
 
         build_script_filename = self.build_script_emitter.filename()
         build_script_contents = self.build_script_emitter.contents(
+            self.name,
             self._get_source_dir_abs_path(),
             self._get_configure_dir_abs_path(),
             self._get_build_dir_abs_path(),
             self.toolchains_manifest,
-            self.toolchain_settings_provider.get_toolchain_settings(),
+            toolchains_settings,
             self.build_types,
             [d.name for d in self.workspace_def.targets],
             targets_impls
@@ -440,6 +486,7 @@ def open_workspace(wks_dir: str, *, dev_mode=False) -> Workspace:
     )
 
     wks = Workspace(
+        name=wks_dir,
         wks_fs=wks_fs,
         logger=logger,
         system_config_provider=system_config_provider,
